import AssertionError from "../Error.mjs";

/**
 * Expects a callback to throw an error.
 *
 * @param {callable} callback
 * @param {mixed} err optional Error object to be instance of, or message
 * @return {bool}
 * @throws {Error} If callback doesn't throw, or throws wrong error/message.
 *
 * @package Tramline
 * @subpackage Assertions
 * @author James Walker
 * @copyright James Walker 2018
 * @license MIT License
 */
let throws = (callback, err=null) => {
	try {
		callback();
		throw new Throw();
	}
	catch (error) {
		let message = error.message;
		if (error instanceof Throw) {
			throw new Error(`Test passed when expected to throw.`);
		}
		else if (err) {
			if (typeof err === "string") {
				if (error.message !== err) {
					throw new AssertionError(`Wrong message.`, message, err);
				}
			}
			else if (!(error instanceof err)) {
				throw new AssertionError(`Wrong throwable.`, error, err);
			}
		}
	}
	return true;
}

/**
 * Dummy error class to detect when callbacks 
 * 	throw when they are not expected to.
 */
class Throw extends Error {}

/**
 * Module exports
 */
export default throws;