import Config from "./Config.mjs";

/**
 * Cli
 *
 * @package Tramline
 * @author James Walker
 * @copyright James Walker 2018
 * @license MIT License
 */
class Cli {

	/**
	 * Create a configuration object.
	 *
	 * Uses arguments from `process.argv` CLI, or passed.
	 *
	 * @param {array} args optional
	 * @return {Config}
	 */
	static config(args=null) {
		return new Config((args ? this.argsProcess(args) : this.args()));
	}


	/**
	 * Get all arguments from `process.argv`.
	 *
	 * Returns an object of arguments.
	 *
	 * `root` is `0`, all other args are named flags.
	 *
	 * @return {array}
	 */
	static args() {
		return this.argsProcess(process.argv.slice(2));
	}


	/**
	 * Process arguments from an array.
	 *
	 * Returns an object of associative argument pairs.
	 *
	 * `root` is `0`, all other args are named flags.
	 *
	 * @return {array}
	 */
	static argsProcess(arr) {
		let args = {};
		args.root = arr[0];
		for (let i = 1; i < arr.length; i = i + 2) {
			args[arr[i].slice(2)] = arr[i + 1];
		}
		return args;
	}

}

export default Cli;