import path from "path";

/**
 * Config
 *
 * @package Tramline
 * @author James Walker
 * @copyright James Walker 2018
 * @license MIT License
 */
class Config {

	/**
	 * Constructor.
	 *
	 * @param {object} config
	 * @return {self}
	 */
	constructor(config={}) {

		/**
		 * Root directory
		 *
		 * @var {string}
		 */
		this.root = null;

		/**
		 * Test files matcher
		 *
		 * @var {string}
		 */
		this.tests = ".test.mjs";

		/**
		 * Configuration save
		 */
		this.save(config);

	}


	/**
	 * Configuration save.
	 *
	 * @param {object} config
	 * @return {void}
	 */
	save(config={}) {
		Object.keys(config).forEach(k => {
			if (k === "root" && typeof config[k] === "string") {
				config[k] = path.resolve(config[k].replace(/\r/g, ""));
			}
			this[k] = config[k];
		});
	}

}

export default Config;