/**
 * Tramline error
 *
 * @package Tramline
 * @author James Walker
 * @copyright James Walker 2018
 * @license MIT License
 */
class TramlineError extends Error {

	/**
	 * Constructor.
	 *
	 * @param {string} message
	 * @param {mixed} value
	 * @param {mixed} expected optional
	 * @return {self}
	 */
	constructor(message, value, expected="(not specified)") {
		super();

		/**
		 * Value
		 *
		 * @var {mixed}
		 */
		this.value = value;

		/**
		 * Expected
		 *
		 * @var {mixed}
		 */
		this.expected = expected;

		/**
		 * Message
		 *
		 * @var {string}
		 */
		this.message = message;

		/**
		 * Name
		 *
		 * @var {string}
		 */
		this.name = this.constructor.name;

		/**
		 * Capture stack trace
		 */
		this.captureStackTrace(message);
	}


	/**
	 * Capture stack trace using best available method.
	 *
	 * @param {string} message
	 * @return {void}
	 */
	captureStackTrace(message) {
		if (typeof Error.captureStackTrace === "function") {
			Error.captureStackTrace(this, this.captureStackTrace);
		}
		else this.stack = (new Error(message)).stack;
	}


	/**
	 * Extend string typecasting with expected/received values.
	 *
	 * @return {string}
	 */
	toString() {
		let received = `Received: ${this.value}`;
		let expected = `Expected: ${this.expected}`;
		return `${expected}\n${received}\n${this.toStringParent()}`;
	}


	/**
	 * Get string from parent.
	 *
	 * @return {string}
	 */
	toStringParent() {
		return super.toString();
	}

}

export default TramlineError;