/**
 * Test hook error
 *
 * @package Tramline
 * @subpackage Errors
 * @author James Walker
 * @copyright James Walker 2018
 * @license MIT License
 */
class TestHookError extends Error {

	/**
	 * Constructor.
	 *
	 * @param {string} unit
	 * @param {string} hook
	 * @return {self}
	 */
	constructor(unit, hook) {

		/**
		 * Construct + message
		 */
		super(`Test hook "${hook}" failed in "${unit}".`);

		/**
		 * Unit
		 *
		 * @var {string}
		 */
		this.unit = unit;

		/**
		 * Hook
		 *
		 * @var {string}
		 */
		this.hook = hook;

		/**
		 * Name
		 *
		 * @var {string}
		 */
		this.name = this.constructor.name;

		/**
		 * Capture stack trace
		 */
		this.captureStackTrace(this.message);

	}


	/**
	 * Capture stack trace using best available method.
	 *
	 * @param {string} message
	 * @return {void}
	 */
	captureStackTrace(message) {
		if (typeof Error.captureStackTrace === "function") {
			Error.captureStackTrace(this, this.captureStackTrace);
		}
		else this.stack = (new Error(message)).stack;
	}

}

export default TestHookError;