/**
 * Timer
 *
 * @package Tramline
 * @author James Walker
 * @copyright James Walker 2018
 * @license MIT License
 */
class Timer {

	/**
	 * Constructor.
	 *
	 * @param {bool} start optional Start automatically (`true`)
	 * @return {self}
	 */
	constructor(start=true) {

		/**
		 * Start (ms)
		 *
		 * @var {int}
		 */
		this.a = null;


		/**
		 * End (ms)
		 *
		 * @var {int}
		 */
		this.b = null;

		this.reset(start);
	}


	/**
	 * Start timer, if not already started.
	 *
	 * @return {void}
	 */
	start() {
		if (this.a === null) {
			this.a = Date.now();
		}
	}


	/**
	 * End timer, if not already ended.
	 *
	 * @return {int} Elapsed time b - a (ms)
	 */
	end() {
		if (this.b === null) {
			this.b = Date.now();
		}
		return (this.b - this.a);
	}


	/**
	 * Get elapsed time to current time (ms)
	 *
	 * @param {bool} end optional Return time b - a (`true`)
	 */
	elapsed(end=false) {
		return end ? (this.b - this.a) : (Date.now() - this.a);
	}


	/**
	 * Reset.
	 *
	 * @param {bool} start optional Restart automatically (`true`)
	 * @return {void}
	 */
	reset(start=true) {
		this.a = null;
		this.b = null;
		if (start) this.start();
	}

}

export default Timer;