/**
 * Built-in method names
 *
 * Are not tested automatically.
 *
 * @var {array}
 */
const RESERVED = [
	"constructor", "hasOwnProperty", "isPrototypeOf",
	"propertyIsEnumerable", "toString", "valueOf", "toLocaleString"
];

/**
 * Unit
 *
 * @package Tramline
 * @author James Walker
 * @copyright James Walker 2018
 * @license MIT License
 */
class Unit {

	/**
	 * Constructor.
	 *
	 * @param {string} title
	 * @return {self}
	 */
	constructor(title) {

		/**
		 * Title
		 *
		 * @var {string}
		 */
		this.title = title;

		/**
		 * Results
		 *
		 * Results are added indexed by method name 
		 * 	after test methods are executed - the results 
		 * 	are the returned values from that test method.
		 *
		 * @var {object}
		 */
		this.results = {};

	}


	/**
	 * Get tests in the unit.
	 *
	 * Returns all methods not prefixed with `__`.
	 *
	 * @return {array}
	 */
	__tests() {
		let obj = this;
		let methods = [];
		do {
			methods = methods.concat(
				Object.getOwnPropertyNames(obj)
			);
		}
		while (obj = Object.getPrototypeOf(obj));

		/**
		 * Filter to remove invalid methods
		 */
		return methods.filter(method => {
			let valid = (method.slice(0, 2) !== "__");
			let validMethod = (typeof this[method] === "function");
			return (valid && validMethod && !RESERVED.includes(method));
		});
	}

}

export default Unit;