import HeaderExistsError from "./errors/HeaderExistsError.mjs";

/**
 * Config store
 *
 * @package Requesty
 * @author James Walker
 * @copyright James Walker 2018
 * @license MIT License
 */
class Config {

	/**
	 * Construct - default config.
	 *
	 * @return {self}
	 */
	constructor() {
		this.urlBase = "";
		this.urlBaseEnabled = true;
		this.defaultHeaders = {};
		this.defaultHeadersEnabled = true;
	}


	/**
	 * Add a new default header, if not already added.
	 *
	 * @param {string} header Header
	 * @param {string} value Value
	 * @return {void}
	 * @throws {HeaderExistsError} If header already exists.
	 */
	addHeader(header, value) {
		if (!this.headerSet(header)) {
			this.setHeader(header, value);
		}
		else throw new HeaderExistsError(header);
	}


	/**
	 * Set default header value, even if already exists.
	 *
	 * @param {string} header Header
	 * @param {string} value Value
	 * @return {void}
	 */
	setHeader(header, value) {
		this.defaultHeaders[header] = value;
	}


	/**
	 * Get default header value.
	 *
	 * @param {string} header
	 * @return {string|null}
	 */
	getHeader(header) {
		return this.defaultHeaders[header];
	}


	/**
	 * Get if a named default header is set.
	 *
	 * @param {string} header
	 * @return {bool}
	 */
	headerSet(header) {
		return this.defaultHeaders.hasOwnProperty(header);
	}

}

export default new Config();