/**
 * Response class
 *
 * Properties:
 * - body - Body data
 * - bodyRaw - Body as plain text
 * - type - Value defining response type
 * - status - Status code
 * - message - Status message
 * - request - Request object
 * - interface - Interface object (e.g. `XMLHttpRequest`)
 * - headers - Object of response headers
 *
 * @package Requesty
 * @author James Walker
 * @copyright James Walker 2018
 * @license MIT License
 */
class Response {

	/**
	 * Constructor.
	 *
	 * @return {self}
	 */
	constructor() {
		this.body = null;
		this.bodyRaw = null;
		this.type = null;
		this.status = null;
		this.message = null;
		this.request = null;
		this.interface = null;
		this.headers = {};
	}


	/**
	 * Parse body as JSON.
	 *
	 * @return {string}
	 */
	get json() {
		return JSON.parse(this.bodyRaw);
	}


	/**
	 * Get a header value.
	 *
	 * @param {string} header
	 * @return {string|null}
	 */
	getHeader(header) {
		return this.headers[header];
	}


	/**
	 * Get whether a header is set.
	 *
	 * @param {string} header
	 * @return {bool}
	 */
	headerSet(header) {
		return this.headers.hasOwnProperty(header);
	}

}

export default Response;