import Unit from "tramline/src/Unit";
import * as tramline from "tramline";
import RequestyError from "../src/errors/RequestyError.mjs";
import {main as Requesty, Request} from "../src/requesty.mjs";

/**
 * Assertions
 *
 * @var {object}
 */
const assertions = tramline.assertions;


/**
 * Requesty tests.
 *
 * @package Requesty
 * @subpackage Tests
 * @author James Walker
 * @copyright James Walker 2018
 * @license MIT License
 */
class RequestyTests extends Unit {

	/**
	 * Requesty instance for tests.
	 *
	 * @return {Requesty}
	 */
	__init() {
		return new Requesty.constructor();
	}


	/**
	 * Construction.
	 *
	 * @param {Requesty} req
	 * @return {void}
	 */
	construction(req) {
		assertions.assert(req.hasOwnProperty("requesty"));
	}


	/**
	 * Making request works.
	 *
	 * Cannot end-to-end test as async/Promise-based.
	 *
	 * Based on individual modules so can manually review and assert 
	 * 	operation by cross-referencing individual unit tests.
	 *
	 * @param {Requesty} req
	 * @return {void}
	 */
	req(req) {
		let response = req.req("http://localhost", {method: "GET"});
		assertions.assert(response instanceof Promise);
	}


	/**
	 * Request submission of object works
	 *
	 * Cannot end-to-end test as async/Promise-based.
	 *
	 * Based on individual modules so can manually review and assert 
	 * 	operation by cross-referencing individual unit tests.
	 *
	 * @param {Requesty} req
	 * @return {void}
	 */
	request(req) {
		let request = new Request("http://localhost");
		assertions.assert(req.request(request) instanceof Promise);
	}


	/**
	 * Throws when request not a Request.
	 *
	 * @param {Requesty} req
	 * @return {void}
	 */
	requestInvalid(req) {
		assertions.throws(() => req.request({}), RequestyError);
	}

}

export default RequestyTests;