import Unit from "tramline/src/Unit";
import * as tramline from "tramline";
import Response from "../src/response.mjs";

/**
 * Assertions
 *
 * @var {object}
 */
const assertions = tramline.assertions;


/**
 * Response test
 *
 * @package Requesty
 * @subpackage Tests
 * @author James Walker
 * @copyright James Walker 2018
 * @license MIT License
 */
class ResponseTest extends Unit {

	/**
	 * Return new Response for each case.
	 *
	 * @return {Response}
	 */
	__run() {
		return new Response();
	}


	/**
	 * Construction tests.
	 *
	 * @param {Response} response
	 * @return {void}
	 */
	construction(response) {
		assertions.assert(response.hasOwnProperty("body"));
		assertions.assert(response.hasOwnProperty("bodyRaw"));
		assertions.assert(response.hasOwnProperty("type"));
		assertions.assert(response.hasOwnProperty("status"));
		assertions.assert(response.hasOwnProperty("message"));
		assertions.assert(response.hasOwnProperty("request"));
		assertions.assert(response.hasOwnProperty("interface"));
		assertions.assert(response.hasOwnProperty("headers"));
		assertions.assert(typeof response.headers === "object");
	}


	/**
	 * JSON parsing of body contents.
	 *
	 * @param {Response} response
	 * @return {void}
	 */
	json(response) {
		let json = "{\"foo\": \"bar\"}";
		response.bodyRaw = json;
		let parsed = response.json;
		assertions.assert(typeof parsed === "object");
		assertions.assert(parsed.hasOwnProperty("foo"));
		assertions.identical(parsed.foo, "bar");
	}


	/**
	 * Header retrieval.
	 *
	 * @param {Response} response
	 * @return {void}
	 */
	getHeader(response) {
		response.headers.foo = "bar";
		assertions.falsey(response.getHeader("foobar"));
		assertions.identical(response.getHeader("foo"), "bar");
	}


	/**
	 * Get whether header is set.
	 *
	 * @param {Response} response
	 * @return {void}
	 */
	headerSet(response) {
		response.headers.foo = "bar";
		assertions.truthy(response.headerSet("foo"));
		assertions.falsey(response.headerSet("foobar"));
	}

}

export default ResponseTest;